﻿using Microsoft.Office.Core;
using Microsoft.Office.Interop.Outlook;
using System;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 리본 컨텍스트 메뉴
    /// </summary>
    [ComVisible(true)]
    public class RibbonContextMenu : IRibbonExtensibility
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 리본 UI
        /// </summary>
        private IRibbonUI ribbonUI;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RibbonContextMenu()

        /// <summary>
        /// 생성자
        /// </summary>
        public RibbonContextMenu()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region (IRibbonExtensibility) 커스텀 UI 구하기 - GetCustomUI(ribbonID)

        /// <summary>
        /// 커스텀 UI 구하기
        /// </summary>
        /// <param name="ribbonID">리본 ID</param>
        /// <returns>커스텀 UI</returns>
        public string GetCustomUI(string ribbonID)
        {
            return GetResourceText("TestProject.RibbonContextMenu.xml");
        }

        #endregion

        #region 커스텀 UI 로드시 처리하기 - customUI_Load(ribbonUI)

        /// <summary>
        /// 커스텀 UI 로드시 처리하기
        /// </summary>
        /// <param name="ribbonUI">리본 UI 인터페이스 객체</param>
        public void customUI_Load(IRibbonUI ribbonUI)
        {
            this.ribbonUI = ribbonUI;
        }

        #endregion
        #region 커스텀 컨텍스트 메뉴 표시 여부 구하기 - customContextMenu_GetVisible(ribbonControl)

        /// <summary>
        /// 커스텀 컨텍스트 메뉴 표시 여부 구하기
        /// </summary>
        /// <param name="ribbonControl">리본 컨트롤</param>
        /// <returns>처리 결과</returns>
        public bool customContextMenu_GetVisible(IRibbonControl ribbonControl)
        {
            string id         = ribbonControl.Id;
            string folderName = ((Folder)ribbonControl.Context).Name;

            if(id == "customContextMenu1" && folderName == "받은 편지함")
            {
                return true;
            }

            if(id == "customContextMenu2" && folderName == "보낼 편지함")
            {
                return true;
            }

            return false;
        }

        #endregion
        #region 커스텀 컨텍스트 메뉴 1 클릭시 처리하기 - customContextMenu1_Click(ribbonControl)

        /// <summary>
        /// 커스텀 컨텍스트 메뉴 1 클릭시 처리하기
        /// </summary>
        /// <param name="ribbonControl">리본 컨트롤</param>
        public void customContextMenu1_Click(IRibbonControl ribbonControl)
        {
            if(ribbonControl.Id == "customContextMenu1")
            {
                System.Windows.Forms.MessageBox.Show("커스텀 컨텍스트 메뉴 1을 클릭했습니다.");
            }
        }

        #endregion
        #region 커스텀 컨텍스트 메뉴 2 클릭시 처리하기 - customContextMenu2_Click(ribbonControl)

        /// <summary>
        /// 커스텀 컨텍스트 메뉴 2 클릭시 처리하기
        /// </summary>
        /// <param name="ribbonControl">리본 컨트롤</param>
        public void customContextMenu2_Click(IRibbonControl ribbonControl)
        {
            if(ribbonControl.Id == "customContextMenu2")
            {
                System.Windows.Forms.MessageBox.Show("커스텀 컨텍스트 메뉴 2을 클릭했습니다.");
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 리소스 텍스트 구하기 - GetResourceText(resourceName)

        /// <summary>
        /// 리소스 텍스트 구하기
        /// </summary>
        /// <param name="resourceName">리소스명</param>
        /// <returns>리소스 텍스트</returns>
        private static string GetResourceText(string resourceName)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();

            string[] resourceNameArray = assembly.GetManifestResourceNames();

            for(int i = 0; i < resourceNameArray.Length; ++i)
            {
                if(string.Compare(resourceName, resourceNameArray[i], StringComparison.OrdinalIgnoreCase) == 0)
                {
                    using(StreamReader resourceReader = new StreamReader(assembly.GetManifestResourceStream(resourceNameArray[i])))
                    {
                        if(resourceReader != null)
                        {
                            return resourceReader.ReadToEnd();
                        }
                    }
                }
            }

            return null;
        }

        #endregion
    }
}